	PROGRAM DFORM

C This program takes runoff output and formats it for a DECUS paper.

C Written by:  J. L. Bingham
C              Mantech International
C              Alexandria, Va
C              (703) 960-8900

C Prepare the RUNOFF source as follows:

C	.page size 60,108
C	.lm 25; .rm 85; .nonumber
C	.c;title
C	.b1;.c;name
C	.c;company
C	.c;city and state
C	.b3;.c;^&ABSTRACT\&
C	.B1;abstract paragraph
C	.b3
C	.page size 10000,52
C	.lm 0;.rm 52
C	.c;^&first heading\&
C	etc,etc,etc

C	Do not exceed the column width (52) in any literal section.

C	The above settings are for output with 12 characters per inch.
C	For 10 characters per inch use 90, 20 and 70 vice 108, 25 and
C	85 for the initial settings and 43 vice 52 after the abstract.
C	Character declarations and format statements should be changed
C	accordingly, but DO NOT REDUCE THE (APPARENT) EXCESS LENGTH
C	of the fields.

	IMPLICIT INTEGER (A-Z)

	CHARACTER*35 IFILE, OFILE, CCONT*1, CRLF*2, CR*1, DEL*1
	CHARACTER*87 HLINE
	CHARACTER*55 LINE (292), SAV1, SAV (10)
	LOGICAL STARTA, LASTP, EMPTY, NEWLINE, INTERACTIVE, TOP
	LOGICAL INTO1, INTO2
	PARAMETER (LINELIM=73)
C HLINE and LINE allow 2 bytes for CRLF and one for an extra CR,
C   LINE allows for non-counting lines (underscoring)

C Define EMPTY and NEWLINE

	EMPTY (SAV1) = INDEX (SAV1, CRLF) .EQ. 1
	NEWLINE (SAV1) = INDEX (SAV1, CRLF) .NE. 0

C Get the name of the input file and open it.  Default extension is .MEM.

	TYPE 601, 'Enter input (RUNOFF output) file name: '
 601	FORMAT ('$', A)
	ACCEPT 501, IFILE
 501	FORMAT (A35)
	DOT = INDEX (IFILE, '.')
	CALL STR$TRIM (IFILE, IFILE, ILEN)
	IF (DOT .EQ. 0) IFILE = IFILE (:ILEN) // '.MEM'
	OPEN (NAME=IFILE, UNIT=1, TYPE='OLD')

C Find out what type of output is desired and open the output file.  If a
C   filename is specified its default extension is .DEC.

	TYPE*, 'Enter "I" for interactive output, or a filename for output to'
	TYPE601,' a file. Default name is same as input--extension is .DEC: '
	ACCEPT 501, OFILE
	IF (OFILE .EQ. 'I' .OR. OFILE .EQ. 'i') THEN

		INTERACTIVE = .TRUE.
		OFILE = 'SYS$OUTPUT'
		TYPE*, 'Output will be to your terminal.'
		TYPE*, '  Enter a CR to start each page.'
		TYPE 201, '1', ' '
		CCONT = ' '
		ACCEPT 502, DUMMY
 502		FORMAT (A4)

	ELSE

		IF (OFILE .EQ. ' ') OFILE = IFILE (:ILEN)
		DOT = INDEX (OFILE, '.')
		CALL STR$TRIM (OFILE, OFILE, ILEN)
		IF (DOT .EQ. 0) OFILE = OFILE (:ILEN) // '.DEC'

	ENDIF
	OPEN (NAME=OFILE, UNIT=2, TYPE='NEW')

C Initialize

	STARTA = .FALSE.
	LASTP = .FALSE.
	TOP = .TRUE.
	QSAV = 0
	LINEH = 1
	BLINEK = 0
	CCONT = ' '

	CRLF = CHAR ('D'X) // CHAR ('A'X)
	CR = CHAR ('D'X)
	DEL = CHAR ('7F'X)

C Do the top of the first page.  This part is done when we have seen the
C   word "ABSTRACT" and later have seen three blank lines.

	WRITE (2,201) CCONT, CCONT	!Start with a blank line
	DO WHILE (BLINEK .LT. 3)

		READ (1, 101) HLINE
 101		FORMAT (A87)
		POS = INDEX (HLINE, CRLF)
		IF (POS .NE. 0) HLINE (POS:POS+1) = '  '
		WRITE (2, 201) CCONT, HLINE
 201		FORMAT (A1, A85)
		IF (POS .NE. 0 .AND. (HLINE .NE. ' ' .OR. .NOT. TOP)) THEN
			!newline and not an excess blank line at the top

			CCONT = ' '
			LINEH = LINEH + 1
			TOP = .FALSE.

		ELSE

			CCONT = '+'

		ENDIF
		IF (INDEX (HLINE, 'ABSTRACT') .NE. 0) STARTA = .TRUE.
		IF (STARTA .AND. HLINE .EQ. ' ') THEN

			BLINEK = BLINEK + 1

		ELSE

			BLINEK = 0

		ENDIF

	ENDDO

C Do the remainder of the paper in two columns.

	DO WHILE (.NOT. LASTP)

		TOPK = 1
		IREC = 1
		LINEK = 1
		LIM = LINELIM - LINEH
		LIMX2 = LIM * 2
		STATE = 0
		INTO1 = .FALSE.
		INTO2 = .FALSE.
		DO WHILE (LINEK .LE. LIMX2) !Read a page into memory

			IF (TOPK .GT. QSAV) THEN

				READ (1, 102, END=900) LINE (IREC)
 102				FORMAT (A55)

			ELSE

				LINE (IREC) = SAV (TOPK)

			ENDIF
			TOPK = TOPK + 1
			OLDSTATE = STATE
			IF (EMPTY (LINE (IREC))) THEN

				IF (STATE .EQ. -2 .OR. STATE .EQ. -1) THEN
					STATE = -1
				ELSE
					STATE = 0
				ENDIF
				DELTA = 1

			ELSE IF (.NOT. NEWLINE (LINE (IREC))) THEN

				IF (STATE .EQ. 0) THEN

					GPTR = IREC
					STATE = -3

				ELSE IF (STATE .EQ. -2 .OR. STATE .EQ.-1)THEN

					GPTR = IREC
					STATE = 0

				ENDIF
				DELTA = 0

			ELSE !Non blank line with CRLF (not a heading since
				!headings are underscored)

				IF (STATE .EQ. 0) THEN

					IF (GPTR .NE. IREC - 1) THEN
						GPTR = IREC
						STATE = 1
					ENDIF

				ELSE IF (STATE .EQ. -2) THEN

					STATE = 2

				ELSE

					STATE = STATE + 1

				ENDIF
				DELTA = 1

			ENDIF
C The meanings of the possible values of STATE are:

C   STATE=-3 => Current line may be a section heading (a line with underscoring
C                  following at least one blank line.
C   STATE=-2 => Current line is probably underscoring for a STATE -3 line.
C   STATE=-1 => GPTR points at the first line of a section heading. The
C                  heading part is considered to continue until there is
C                  a line of information.
C   STATE=0  => the current line is blank or is the first line of
C                  information in a heading.
C   STATE=1  => the current line is the first nonblank line after one
C                  or more STATE 0 lines.
C   STATE=2 or more => the value usually indicates the number of contiguous
C                  non blank lines in the current section.  GPTR points
C                  at the top line in the group

d	type*,STATE,linek,irec,empty(line(irec)),newline(line(irec)),gptr
d	type*,line(irec)
			IF(LINEK .EQ. 1 .AND. EMPTY(LINE(IREC))) THEN
				!Get rid of blank lines at top of the LH column

d	type*,'In lh blank line revoval'
				DELTA = 0
				IREC = IREC - 1

			ELSE IF (LINEK .EQ. LIM+1) THEN

				IF (.NOT. INTO1) RHK = IREC
				IF (STATE .EQ. -1 .OR. STATE .EQ. -2)THEN

d	type*,'In heading to rh col section'
					DO I = IREC - GPTR, 0, -1
						!Do not leave a heading hanging
						!at bottom of LH column

						LINE(IREC+I)=LINE(GPTR+I)
						LINE (GPTR+I) = CRLF
						IF(NEWLINE(LINE(IREC+I)))
	1						LINEK = LINEK + 1

					ENDDO
					IREC = 2 * IREC - GPTR
					LINEK = LINEK - 1
					STATE = 10
					INTO1 = .TRUE.

				ELSE IF (EMPTY (LINE (IREC))) THEN
					!Get rid of blank lines at top of RH

d	type*,'In rh col blank line removal'
					DELTA = 0
					IREC = IREC - 1
					INTO1 = .TRUE.
					STATE = 10

				ELSE IF (STATE .EQ. 2) THEN
					!Move a widow from bot of LH to top RH

d	type*,'Moving widow to top of rh col'
					ILIM = 1
					INTO1 = .TRUE.
					IF (INDEX(LINE(IREC-1),'_').NE.0)ILIM=2
					DO I = ILIM, 0, -1

						J = IREC + I - ILIM
D	type*,'line(',j,') to line(',irec+i,'),',line(j)
						LINE(IREC+I)=LINE(J)
						IF(NEWLINE(LINE(IREC+I)))THEN

							LINE(J) = CRLF
							LINEK = LINEK+1
						ELSE

							LINE(J) = ' '

						ENDIF

					ENDDO
					IREC = IREC + ILIM

				ENDIF

			ELSE IF(LINEK.EQ.LIM+2.AND..NOT.INTO2.AND..NOT.INTO1
	1			.AND.OLDSTATE.GE.2.AND.EMPTY(LINE(IREC)))THEN
				!Move a line from bottom of LH to top of RH to
				!prevent widow on top of RH. Move two if
				!moving one would leave a widow at bottom of LH
d	type*,'lim+2 if,irec=',irec,',linek=',linek,',STATE=',STATE,oldSTATE
d	type*,'line(irec)=',line(irec)
				INTO2 = .TRUE.
				ILIM = 2
				IF (OLDSTATE .EQ. 3) ILIM = 3
				LINEK = LINEK + ILIM - 1
				IF (.NOT. NEWLINE (LINE (IREC-ILIM-1)))
	1				ILIM = ILIM + 1
				DO I = ILIM, 1, -1
D	TYPE*,'IREC,I,ILIM,OLDACT,ACT',IREC,I,ILIM,OLDSTATE,STATE
d	type*,'line',IREC+I-1-ILIM,':',line(IREC+I-1-ILIM)
d	type*,'to line', irec+i-2,':',line(irec+i-2)

					LINE(IREC+I-2) =
	1					LINE(IREC+I-ILIM-1)
					LINE (IREC+I-ILIM-1)=CRLF

				ENDDO
				IREC = IREC + ILIM - 1
				LINE (IREC) = CRLF

			ELSE IF (LINEK .EQ. LIMX2) THEN

				DELTA = 1
				IF (STATE .EQ. -1 .OR. STATE .EQ. -2 .OR.
	1				STATE .EQ. -3) THEN
					!Do not leave heading at bottom of RH

d	type*,'Moving heading to next page'
					LINEK = LINEK + 1
					QSAV = 0
					DO I = GPTR, IREC

						QSAV = QSAV + 1
						SAV(QSAV) = LINE (I)
						LINE (I) = CRLF

					ENDDO

				ELSE

					READ (1, 102, END=800) SAV (1)
					READ (1, 102, END=800) SAV (2)
					IF (.NOT. EMPTY (SAV (1)) .AND.
	1 					(STATE .EQ. 1 .OR.
	1					STATE .GE. 2 .AND.
	1					EMPTY (SAV (2)))) THEN
						!Widow handling between pages

d	type*,'moving line from one page to next to avoid widow'
						IF (STATE .NE. 2) THEN

							SAV (3) = SAV (2)
							SAV (2) = SAV (1)
							SAV (1) = LINE (IREC)
							QSAV = 3

						ELSE

							SAV (4) = SAV (2)
							SAV (3) = SAV (1)
							SAV (2) = LINE (IREC)
							SAV (1) = LINE(IREC-1)
							QSAV = 4

						ENDIF
						LINE (IREC) = CRLF

					ELSE

						QSAV = 2

					ENDIF

				ENDIF

			ENDIF
			LINEK = LINEK + DELTA
			IREC = IREC + 1

		ENDDO

		GOTO 1000

 800		CONTINUE
		IF (INTERACTIVE) GOTO 900
		TYPE*, 'End of file with less than three lines on last page'
 900		CONTINUE
		LASTP = .TRUE.
		RHK = IREC / 2 + 1
		IF (.NOT. NEWLINE (LINE(RHK-1))) RHK = RHK - 1

1000		DO I = IREC, MIN (IREC+10, 288)

			LINE (I) = CRLF

		ENDDO
		LHK = 1
		LLIM = RHK
		RLIM = IREC

		LCRLF = INDEX (LINE (LHK), CRLF)
		IF (LCRLF .NE. 0) LINE (LHK)(LCRLF:LCRLF+1) = '  '

		RCRLF = INDEX (LINE (RHK), CRLF)
		IF (RCRLF .NE. 0) LINE (RHK)(RCRLF:RCRLF+1) = '  '
		IF (LINE (RHK)(1:1) .EQ. CR) LINE (RHK)(1:1) = DEL

		DO WHILE (LHK .LT. LLIM .OR. RHK .LT. RLIM) !Output the page

			WRITE (2, 202) CCONT, LINE(LHK), LINE(RHK)
 202			FORMAT (A1, A53, T58, A53)

			CCONT = '+'	!Assume overprinting
			LHK = LHK + 1
			RHK = RHK + 1

			IF (LCRLF .EQ. 0 .AND. RCRLF .EQ. 0) THEN
				!Overprint both sides

			ELSE IF (RCRLF .EQ. 0) THEN
				!Overprint the right hand side

				LHK = LHK - 1
				LINE (LHK) = CRLF

			ELSE IF (LCRLF .EQ. 0) THEN
				!Overprint the left hand side

				RHK = RHK - 1
				LINE (RHK) = CRLF

			ELSE !No overprinting

				CCONT = ' '

			ENDIF

			LCRLF = INDEX (LINE (LHK), CRLF)
			IF (LCRLF .NE. 0) LINE (LHK)(LCRLF:LCRLF+1) = '  '

			RCRLF = INDEX (LINE (RHK), CRLF)
			IF (RCRLF .NE. 0) LINE (RHK)(RCRLF:RCRLF+1) = '  '
			IF (LINE (RHK)(1:1) .EQ. CR) THEN

				IF (LINE (RHK)(2:2) .EQ. ' ' .AND.
	1				LINE (RHK)(53:53) .EQ. '_') THEN

					LINE (RHK)(1:1) = DEL !Try ' ' if some
						!underlining is offset to left.

				ELSE

					LINE (RHK)(1:1) = DEL

				ENDIF

			ENDIF

		ENDDO

		CCONT = '1'
		LINEH = 0
		IF (INTERACTIVE) THEN

			WRITE (2, 202) CCONT
			CCONT = ' '
			ACCEPT 502, DUMMY

		ENDIF

	ENDDO

 	CLOSE (UNIT = 1)
	CLOSE (UNIT = 2)

	IF (.NOT. INTERACTIVE) TYPE*, 'Output file is ',OFILE
	STOP

	END
