(**********************************************************
These string and other usefull PASCAL routines were adapted 
from the PASCAL USER GROUP (PUG) PASCAL NEWS for VAX PASCAL
by 



Francis J. Monaco
Major, US ARMY
Systems Manager, Computer Graphics Laboratory
Department of Geography and Computer Science
The United States Military Academy
West Point, New York 10996
914-938-2063

These routines can be included in a VAX PASCAL program 
as long as a the following GLOBAL variables are declared
at the main lex level:

CONST
STRINGMAX = 131;
BLANK = ' ';
TYPE
STRING = RECORD
CH : PACKED ARRAY [ 1 .. STRINGMAX ] OF CHAR;
LEN : 0 .. STRINGMAX
END;
CHARS = PACKED ARRAY [ 1 .. STRINGMAX ] OF CHAR;
*****************************************************************)

FUNCTION UPPER ( CH : CHAR ): CHAR;
BEGIN
 IF CH IN ['a' .. 'z'] 
   THEN
     UPPER := CHR ( ORD ( CH ) - 32 )
   ELSE
     UPPER := CH;
END;

(*****************************************************************)

FUNCTION LOWER ( CH : CHAR ): CHAR;
BEGIN
 IF CH IN ['A' .. 'Z']
  THEN
    LOWER := CHR ( ORD ( CH ) + 32 )
  ELSE
    LOWER := CH;
END;

(**************************************************************)

PROCEDURE GOTOXY ( X, Y : INTEGER );
(* Smart GOTOXY Routine for hazeltine 1500 CRT *)
(* Requires x, y of type integer : see code for functions *)
(* Note that x = -1 signals a terminal dependent function;
otherwise, cursor control to X, Y is assumed *)
(*
For terminal functions, call GOTOXY with (-1 and...
1	Homecursor
2	Clearscreen
3	Clearforeground
4	Cleartoeol
5	Cleartoeos
6	Clearbackground
7	Background follows (light intensity)
8	Foreground follows (dark intensity)
*)
(* F. J. Monaco
22 Oct 1980 *)

CONST 
      LEADINTOSCREEN = 126; (* the tilde character on the Hazeltine 1500*)
      DIRECTCURSOR = 17;
      HOMECURSOR = 18;
      CLEARFOREGROUND = 29;
      CLEARTOEOL = 15;
      CLEARTOEOS = 24;
      CLEARBACKGROUND = 23;
      BACKGROUNDFOLLOWS = 25;
      FOREGROUNDFOLLOWS = 31;
      CLEARSCREEN = 28;

VAR 
    ROW, COLUMN : INTEGER;
    ZAPCURSOR : PACKED ARRAY [ 0 .. 3 ] OF CHAR;
    ZAPSCREEN : PACKED ARRAY [ 0 .. 1 ] OF CHAR;
    COUNTER : INTEGER;
BEGIN
 IF ( ( Y >= 0 ) AND ( X >= 0 ) )
  THEN
   BEGIN
    IF X <= 30
     THEN
      COLUMN := X + 96
     ELSE
      COLUMN := X;
    ROW := Y + 96;
    ZAPCURSOR [ 0 ] := CHR ( LEADINTOSCREEN );
    ZAPCURSOR [ 1 ] := CHR ( DIRECTCURSOR );
    ZAPCURSOR [ 2 ] := CHR ( COLUMN );
    ZAPCURSOR [ 3 ] := CHR ( ROW );
    FOR COUNTER := 0 TO 3 DO
     WRITE ( ZAPCURSOR [ COUNTER ] )
   END
  ELSE
   BEGIN
    IF X = -1
     THEN
      BEGIN
       ZAPSCREEN [ 0 ] := CHR ( LEADINTOSCREEN );
       CASE Y OF
        1:  ZAPSCREEN [ 1 ] := CHR ( HOMECURSOR );
        2:  ZAPSCREEN [ 1 ] := CHR ( CLEARSCREEN );
        3:  ZAPSCREEN [ 1 ] := CHR ( CLEARFOREGROUND );
        4:  ZAPSCREEN [ 1 ] := CHR ( CLEARTOEOL );
        5:  ZAPSCREEN [ 1 ] := CHR ( CLEARTOEOS );
        6:  ZAPSCREEN [ 1 ] := CHR ( CLEARBACKGROUND );
        7:  ZAPSCREEN [ 1 ] := CHR ( BACKGROUNDFOLLOWS );
        8:  ZAPSCREEN [ 1 ] := CHR ( FOREGROUNDFOLLOWS )
       END;
       IF ( (Y >= 1) AND (Y <= 8 ) )
        THEN
         FOR COUNTER := 0 TO 1 DO
          WRITE ( ZAPSCREEN [ COUNTER ] );
      END;
   END;
END   (* GOTOXY *) ;
(************************************************************
STRING FUNCTIONS and PROCEDURES
*************************************************************)

PROCEDURE FILLC ( VAR S : CHARS );

VAR 
    I : INTEGER;
BEGIN
 FOR I := 1 TO STRINGMAX DO
  S [I] := BLANK;
END;
(*******************************************************************)

FUNCTION LENGTH (S: STRING): INTEGER;
BEGIN
 LENGTH := S.LEN;
END;
(********************************************************************)

PROCEDURE FILLCHAR ( VAR S: STRING; CHARACTER: CHAR);

VAR 
    I : INTEGER;
BEGIN
 S . LEN := 0;
 FOR I := 1 TO STRINGMAX DO
  S . CH [ I ] := CHARACTER;
END;
(*********************************************************************)

PROCEDURE CONCAT ( VAR S : STRING; T : STRING );

VAR 
    I, J : INTEGER;
BEGIN
 IF S . LEN + T . LEN > STRINGMAX
  THEN
   J := STRINGMAX - S . LEN
  ELSE
   J := T . LEN;
 FOR I := 1 TO J DO
  S . CH [ S . LEN + I ] := T . CH [ I ];
 S . LEN := S . LEN + J;
END;
(**********************************************************************)

FUNCTION POS ( S, T : STRING ): INTEGER;

VAR 
    I, J : INTEGER;
    UNEQ : BOOLEAN;
BEGIN
 IF ( ( 1  + T . LEN) > (S . LEN + 1) ) OR (T . LEN = 0)
  THEN
   POS := 0
  ELSE
   BEGIN
    I := 0;
    REPEAT
     I := SUCC ( I );
     J := 0;
     REPEAT
      J := SUCC ( J );
      UNEQ := T . CH [ J ] <> S . CH [ I + J - 1 ];
     UNTIL UNEQ OR ( J = T . LEN );
    UNTIL (NOT UNEQ) OR (I = S . LEN - T . LEN + 1);
    IF UNEQ
     THEN
      POS := 0
     ELSE
      POS := I;
   END;
END;
(******************************************************************)

PROCEDURE READS ( VAR F: TEXT; VAR S: STRING );

VAR 
    CH1 : CHAR;
BEGIN
 FILLCHAR ( S, BLANK );
 WITH S DO
  WHILE ( (NOT EOLN(F)) AND (LEN <=STRINGMAX)) DO
   BEGIN
    LEN := SUCC (LEN);
    READ (F, CH1);
    CH [ LEN ] := CH1;
   END;
 READLN(F);
END;
(********************************************************************)

PROCEDURE WRITES(VAR F: TEXT; S: STRING);

VAR 
    I : INTEGER;
BEGIN
 FOR I := 1 TO S . LEN DO
  WRITE ( F, S . CH [ I ] );
END;
(********************************************************************)

PROCEDURE COPY (VAR T: STRING; S: STRING; START, SPAN: INTEGER);

VAR 
    I: INTEGER;
BEGIN
 IF SPAN < 0
  THEN
   BEGIN
    SPAN := - SPAN;
    START := START - SPAN;
   END;
 IF START < 1
  THEN
   BEGIN
    SPAN := SPAN + START - 1;
    START := 1;
   END;
 IF START + SPAN > S . LEN + 1
  THEN
   SPAN := S . LEN - START + 1;
 IF SPAN <= 0
  THEN
   FILLCHAR ( T, BLANK )
  ELSE
   BEGIN
    FOR I := 1 TO SPAN DO
     T . CH [ I ] := S . CH [ START + I - 1 ];
    FOR I := SPAN + 1 TO STRINGMAX DO
     T . CH [ I ] := ' ';
    T . LEN := SPAN;
   END;
END;
(*******************************************************************)

PROCEDURE DELETE (VAR S: STRING; START, SPAN: INTEGER);

VAR 
    I, LIMIT : INTEGER;
BEGIN
 IF SPAN < 0
  THEN
   BEGIN
    SPAN := - SPAN;
    START := START - SPAN;
   END;
 LIMIT := START + SPAN;
 IF START < 1
  THEN
   START := 1;
 IF LIMIT > S . LEN + 1
  THEN
   LIMIT := S . LEN + 1;
 SPAN := LIMIT - START;
 IF SPAN > 0
  THEN
   BEGIN
    FOR I := 0 TO S . LEN - LIMIT DO
     S . CH [ START + I ] := S . CH [ LIMIT + I ];
    FOR I := S . LEN - SPAN + 1 TO S . LEN DO
     S . CH [ I ] := ' ';
    S . LEN := S . LEN - SPAN;
   END;
END;
(*****************************************************************)

PROCEDURE INSERT (VAR S: STRING; T : STRING; P: INTEGER);

VAR 
    I, J: INTEGER;
BEGIN
 IF T . LEN > 0
  THEN
   IF (P > 0) AND (P <= S . LEN + 1)
    THEN
     BEGIN
      IF S . LEN + T . LEN <= STRINGMAX
       THEN
        S . LEN := S . LEN + T . LEN
       ELSE
        S . LEN := STRINGMAX;
      FOR I := S . LEN DOWNTO P + T . LEN DO
       S . CH [ I ] := S . CH [ I - T . LEN ];
      IF S . LEN < P  + T . LEN
       THEN
        J := S . LEN
       ELSE
        J := P + T . LEN - 1;
      FOR I := P TO J DO
       S . CH [ I ] := T . CH [ I - P + 1 ];
     END
END;
(****************************************************************)

FUNCTION GETINTAT (X, Y : INTEGER): INTEGER;

TYPE 
     STRING = PACKED ARRAY [1..80] OF CHAR;

VAR 
    S : STRING;
    NUM, PWRTEN, I, DIGIT, IMAX: INTEGER;
    GOTINT, BAD, SIGNED, NEGATIVE: BOOLEAN;
BEGIN
 GOTINT := TRUE;
 SIGNED := FALSE;
 REPEAT
  GOTOXY(-1,8);
  GOTOXY(X,Y);
  GOTOXY(-1,4);
  IF NOT GOTINT
   THEN
    BEGIN
     GOTOXY(25,19);
     GOTOXY(-1,4);
     WRITELN('*** Integer Expected ***', CHR(7) );
     GOTOXY(25,21);
     WRITELN('***  PLEASE RETYPE ***');
     GOTINT := TRUE;
    END;
  GOTOXY(X,Y);
  GOTOXY(-1,4);
  WRITE('==>');
  GOTOXY(-1,7);
  READLN(S);
  I := 1;
  WHILE S[I] <> ' ' DO
   I := SUCC ( I );
  IMAX := PRED ( I );
  FOR I := 1 TO IMAX DO
   BEGIN
    BAD := NOT ( ( S[I] IN ['0'..'9']) OR (S[1] IN ['-','+']) );
    IF BAD
     THEN
      GOTINT := FALSE;
   END
 UNTIL GOTINT;
 NUM := 0;
 PWRTEN := 1;
 SIGNED := S[1] IN ['-','+'];
 IF NOT SIGNED
  THEN
   BEGIN
    WHILE ( (IMAX>=1) AND (NUM <MAXINT) ) DO
     BEGIN
      DIGIT := ORD(S[IMAX]) - ORD ('0');
      NUM := NUM + DIGIT * PWRTEN;
      PWRTEN := PWRTEN * 10;
      IMAX := PRED ( IMAX );
     END;
    GETINTAT := NUM;
   END
  ELSE
   BEGIN
    NEGATIVE := S[1] = '-';
    IMAX := PRED ( IMAX );
    WHILE ( (IMAX>=1) AND (NUM < MAXINT) ) DO
     BEGIN
      DIGIT := ORD (S[IMAX]) - ORD ('0');
      NUM := NUM + DIGIT * PWRTEN;
      PWRTEN := PWRTEN * 10;
      IMAX := PRED ( IMAX );
     END;
    IF NEGATIVE
     THEN
      GETINTAT := - NUM
     ELSE
      GETINTAT := NUM;
   END;
END; (*Function getintat*)
